/*
 * Blackcore Networking
 * source@source.blackcore.net
 * Compile: gcc -o bwpd bwpd.c -lpcap
 * Usage: bwpd <libpcap(tcpdump) type filter>
 *
 * Show rate for SYN packets (quotes required for this one, note & symbols)
 * ./bwpd 'tcp[13] & 0x02 != 0 && tcp[13] & 0x30 = 0'
 *
 * Show usage on eth1
 * export DEV=eth1
 * ./bwpd ip
 *
 * Change timeout value to display transfer rate for 5 seconds
 * export BWPD_TIMEOUT=5
 * 
 * Source code is sloppy, I have modified it so many times that it
 * may be poorly optimized.
 *
 * I have only tested this on Linux, I gave up for now on the Windows source,
 * so just ignore that.
 *
 * 05-14-2006
 * - Updated some source code where it was not displaying an error message
 * - slink
 * 03-07-2007
 * - Changed snaplen to 65535 (originally BUFSIZ (8192 under linux)), now will reflect a more accurate transfer rate
 * - slink
 * 12-11-2008
 * - Changed header.caplen to header.len
 * - Changed variable 'int len' to 'unsigned long long' along with various other variables.
*/
/*
    Copyright 2005-2008 blackcore networking
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pcap.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
extern int errno;

#define MIN 10
#define MAX 25
#define SHOWBITS
int  gbs=0,timeout=1,timee,panic=0;
unsigned long long len = 0;
int ss,ee;
unsigned long long nn=0, tb=0, tp=0, count;
float getbw(unsigned long long,char*);
char *dev;
void sig_init(int sig) {
	int tt,a;
	char type[8];
	float rate,b;
	memset(type,0,sizeof(type));
	if (sig == 2 || sig == 3) {
		char type2[8];
		memset(type2,0,sizeof(type2));
		ee=time(NULL);
		tt=ee-ss;
		if (!tt) {
			printf("need to run longer\n");
			exit(1);
		}
		rate =getbw(tb, (char*)&type);
		b = getbw(tb / tt,(char *)&type2);
		a = nn / tt;
		printf("timeout=%u seconds=%u, pkts=%llu total=%.2f%s rate=%.2f%s/s pps=%d/s\n", timeout, tt, nn, rate, type, b, type2, a);
		if (sig != 3) exit(0);
	} else if (sig == 14) {
		tb=tb+len;
#ifndef WIN32
		alarm(timeout);
#endif
		rate=getbw(len,(char*)&type);
		tp+=(count/(unsigned long long)timeout);
		printf("pps=%llu rate=%.2f%s", (unsigned long long)((unsigned long long)count / timeout), rate, type);
#ifdef SHOWBITS
		printf(" bits=%llu",(len * 8LLU)/1024LLU);
#endif
	printf("\n");
	fflush(stdout);
//	printf("count %d/pps - %.2f%s/s\n",count / timeout,rate,type,timeout);
		count=0;
		len=0;
		timee=time(NULL);
	} else {
		printf("sig %d\n",sig);
	}
}
int main(int argc, char *argv[]) {
	char errbuf[PCAP_ERRBUF_SIZE];
	int res;
	struct bpf_program filter;
	struct pcap_pkthdr header;
	const u_char *packet;
	char filter_app[256];
	bpf_u_int32 mask;
	bpf_u_int32 net;
	pcap_t *handle;
	signal(3,&sig_init);
	if (argc<2) return 0;
	if (argc==3) timeout=atoi(argv[2]);
	if (getenv("BWPD_TIMEOUT"))
		timeout = atoi(getenv("BWPD_TIMEOUT"));
	else
		timeout = 1;
	if (getenv("DEV"))
		dev = getenv("DEV");
	else {
		dev = (char *)malloc(32);
		memset(dev,0,32);
		strncpy(dev,"eth0",32);
	}
	fprintf(stderr,"interface: %s\n",dev);
	memset(filter_app, 0, sizeof(filter_app));

	for(res=1;res<argc;res++) {
		strncat(filter_app, argv[res], sizeof(filter_app));
		strncat(filter_app, " ", sizeof(filter_app));
	}
	filter_app[strlen(filter_app)-1] = 0;

	pcap_lookupnet(dev,&net,&mask,errbuf);
//	if ((handle = pcap_open_live(dev,BUFSIZ,1,0,errbuf)) == NULL) {
	if ((handle = pcap_open_live(dev,128,1,0,errbuf)) == NULL) {
		fprintf(stderr, "pcap_open_live: %s\n", strerror(errno));
		return 1;
	}
	pcap_compile(handle,&filter,filter_app,0,net);
	if (pcap_setfilter(handle,&filter) == -1) {
		fprintf(stderr, "pcap_setfilter: %s\n", strerror(errno));
		return 3;
	}

#ifndef WIN32
	signal(14,sig_init);
	alarm(timeout);
#endif
	signal(2,&sig_init);
	timee=time(NULL);
	ss=time(NULL);

	for(;;) {
//		res = pcap_read(handle, &header, packet);

		packet = pcap_next(handle,&header);
		nn+=1;
		count++;
		len+=header.len;
	}
	pcap_close(handle);
	return(0);
}
float getbw(unsigned long long tb, char *type) {
	float rate =0.00;
		if (tb > (1024ULL * 1024ULL * 1024ULL)) {
			rate = (float) tb / 1024LLU /1024ULL /1024ULL;
			strcpy(type, "GB");
		} else if (tb > (1024ULL * 1024ULL)) {
			rate = (float) tb / 1024ULL / 1024ULL;
			strcpy(type, "MB");
		} else if (tb > 1024ULL) {
			rate = (float) tb / 1024ULL;
			strcpy(type, "KB");
		} else {
			rate = (float) tb;
			strcpy(type, "BY");
		}
	return rate;
}
